import datetime
import json
import os
import sys
import urllib.request

client_id = 'clientrequestid=a374dc26-3872-424a-82a6-7577f03bbb15'
req_params = client_id + '&format=json&NoIPv6=true'
version_url = 'https://endpoints.office.com/version/Worldwide?' + req_params
endpoints_url = 'https://endpoints.office.com/endpoints/worldwide?' + req_params

allowed_categories = ['Optimize', 'Allow']

status_file = os.getcwd() + '/current_version.txt'
ip_output_file = os.getcwd() + '/iplist.txt'
log_file = os.getcwd() + '/update_o365_ips.log.txt'

def read_file(filepath):
   if not os.path.isfile(filepath):
      return False
   f = open(filepath, "r")
   contents = f.read()
   f.close()
   return contents

def write_to_file(filepath, contents, overwrite):
   param = "w" if overwrite else "a"
   f = open(filepath, param)
   f.write(contents)
   f.close()

def download_json(url):
   response = urllib.request.urlopen(url)
   raw = response.read()
   parsed = json.loads(raw.decode('utf-8'))
   return parsed

def check_status():
   new_ver_json = download_json(version_url)
   new_ver = False
   if isinstance(new_ver_json, dict) and 'latest' in new_ver_json:
      new_ver = new_ver_json['latest']
   current_ver = read_file(status_file)
   if new_ver:
      msg = datetime.datetime.now().isoformat() + ' ' +  new_ver + '\n'
      write_to_file(log_file, msg, False)
      if not current_ver or current_ver != new_ver:
         write_to_file(status_file, new_ver, True)
         return True
   return False

def generate_ip_list(in_json):
   ip_list = []
   for entry in in_json:
      if 'category' in entry and entry['category'] not in allowed_categories:
         continue
      if 'ips' in entry:
         ip_list.extend(entry['ips'])
   return ip_list

def print_ips(ip_list):
   output = "\n".join(ip_list)
   write_to_file(ip_output_file, output, True)

if not check_status():
   sys.exit(0)
endpoint_list = download_json(endpoints_url)
ip_list = generate_ip_list(endpoint_list)
print_ips(ip_list)

